require 'optparse'
require 'ostruct'
require 'find'
require 'digest/md5'
require 'ftools'

options = OpenStruct.new
opts = OptionParser.new do |opts|
	opts.on("-t", "--tool=TOOLNAME", "Name of the Tool to view the files.") do |t|
		options.tool = t
	end
	opts.on("-s", "--srcfolders FOLDER1,FOLDER2", Array, "The folders which contain the files.") do |s|
		options.srcFolders = s
	end
	opts.on("-d", "--dstfolders FOLDER:CODE", Array, "The folders where the classified files will be copied to.") do |d|
		options.dstFolders = d
	end
	opts.on("-p", "--pattern PATTERN1,PATTERN2", Array, "Patterns the files shall match.") do |p|
		options.patterns = p
	end
	opts.on_tail("-h", "--help", "Show this usage statement.") do |h|
		puts opts
	end
end

begin
	opts.parse!(ARGV)
	if (options.srcFolders == nil || options.dstFolders == nil || options.patterns == nil) 
		raise Exception, "You have to specify at least one source folder, one destination folder and one file pattern"
	end
	
	options.srcFolders.each { |f| 
		if (!File.directory? f)
			puts "Directory #{f} doesn't exist or isn't a directory"
			exit
		end
	}
rescue Exception => e
	puts e, "", opts
	exit
end

dstHash = Hash.new
options.dstFolders.each { |d|
	folderData = d.split ':'
	if (folderData.size == 1 or folderData[1] == "")
		puts "You have to specify a Code for the folder #{d}"
		exit
	end
	if (dstHash[folderData[1]] != nil)
		puts "Error: duplicate Directory Code #{folderData[1]}"
		exit
	end
	if (folderData[1] =~ /\W/)
		puts "Error: Code has to be a word character (letter, number, or underscore)"
		exit
	end
	Dir.mkdir folderData[0] if (!File.directory? folderData[0])
	dstHash[folderData[1]] = folderData[0]
}
	
options.patterns.map! { |x| 
	x.tr!('.', '\.')
	x.tr!('*', '.*')
}
options.patterns.collect! {|x| x + "|" }
options.patterns[options.patterns.size-1] = options.patterns[options.patterns.size-1].chop!
pattern = Regexp.compile(options.patterns.to_s)

fileList = Array.new
options.srcFolders.each {|f|
	Find.find(f) { |path| 
		fileList << path if (!File.directory? path and pattern.match path)
	}
}

separator = "\\"
protocol = OpenStruct.new
protocol.changed = "Changed Files:\n"
protocol.copied = "Copied Files:\n"
protocol.missing = "Missing Files:\n"
protocol.failed = "Failed to edit:\n"
fileList.each { |path|
	if (!File.file? path)
		protocol.missing << "\t" + path + "\n"
		next
	end
	digestOld = Digest::MD5.hexdigest(File.read(path))
	`#{options.tool} "#{path.tr('/', '\\\\')}"`
	if ($?.to_i != 0)
		protocol.failed << "\t" + path +"\n"
		next
	end
	digestNew = Digest::MD5.hexdigest(File.read(path))
	
	protocol.changed << "\t" + path << "\n" if(digestOld != digestNew)
	print "Please enter the code(s) for #{path}:"
	codes = gets
	codes.scan( /./ ) { |c| 
		if (dstHash[c] == nil)
			puts "ignoring invalid code: #{c}"
			next
		end
			
		name = File.basename(path)
		newPath = dstHash[c] + "/" + name
		identical = false
		if (File.file? newPath) 
			digestExisting = Digest::MD5.hexdigest(File.read(newPath))
			identical = digestNew == digestExisting
		end

		if (!identical)
			while (File.file? newPath)
				name = "copy of " + name
				newPath = dstHash[c] + "/" + name
			end
			
			File.copy("#{path}","#{newPath}")
			protocol.copied << "\tfrom #{path} to #{newPath}\n"
		end
		
	}
}

puts protocol.changed
puts protocol.copied
puts protocol.missing
puts protocol.failed
